@@ -106,8 +106,7 @@ std::string DescriptorFullName(const DescriptorType* desc, bool is_internal) {
 std::string ConstantNamePrefix(absl::string_view classname) {
   bool is_reserved = false;
 
-  std::string lower(classname);
-  std::transform(lower.begin(), lower.end(), lower.begin(), ::tolower);
+  std::string lower = absl::AsciiStrToLower(classname);
 
   is_reserved = IsReservedName(lower);
 
@@ -513,7 +512,7 @@ std::string BinaryToPhpString(const std::string& src) {
     if (src[i] == '\\' || src[i] == '$' || src[i] == '"') {
       dest += '\\';
       dest += src[i];
-    } else if (std::isprint(src[i])) {
+    } else if (absl::ascii_isprint(src[i])) {
       dest += src[i];
     } else {
       dest += '\\';
@@ -577,7 +576,7 @@ void GenerateFieldAccessor(const FieldDescriptor* field, const Options& options,
           : "";
 
   // Emit getter.
-  if (oneof != NULL) {
+  if (oneof != nullptr) {
     printer->Print(
         "public function get^camel_name^()\n"
         "{\n"
@@ -713,7 +712,7 @@ void GenerateFieldAccessor(const FieldDescriptor* field, const Options& options,
                    UnderscoresToCamelCase(field->cpp_type_name(), true));
   }
 
-  if (oneof != NULL) {
+  if (oneof != nullptr) {
     printer->Print("$this->writeOneof(^number^, $var);\n", "number",
                    IntToString(field->number()));
   } else if (field->is_repeated()) {
@@ -1068,6 +1067,9 @@ void GenerateHead(const FileDescriptor* file, io::Printer* printer) {
   printer->Print(
       "<?php\n"
       "# Generated by the protocol buffer compiler.  DO NOT EDIT!\n"
+      "# NO CHECKED-IN"
+      // Intentional line breaker.
+      " PROTOBUF GENCODE\n"
       "# source: ^filename^\n"
       "\n",
       "filename", file->name());
@@ -1385,38 +1387,30 @@ static std::string EscapePhpdoc(absl::string_view input) {
   std::string result;
   result.reserve(input.size() * 2);
 
-  char prev = '*';
+  char prev = '\0';
 
   for (std::string::size_type i = 0; i < input.size(); i++) {
     char c = input[i];
     switch (c) {
-      case '*':
-        // Avoid "/*".
-        if (prev == '/') {
-          result.append("&#42;");
-        } else {
-          result.push_back(c);
-        }
-        break;
+      // NOTE: "/*" is allowed, do not escape it
       case '/':
-        // Avoid "*/".
+        // Escape "*/" with "{@*}".
         if (prev == '*') {
-          result.append("&#47;");
+          result.pop_back();
+          result.append("{@*}");
         } else {
           result.push_back(c);
         }
         break;
       case '@':
-        // '@' starts phpdoc tags including the @deprecated tag, which will
-        // cause a compile-time error if inserted before a declaration that
-        // does not have a corresponding @Deprecated annotation.
-        result.append("&#64;");
+        // '@' starts phpdoc tags. Play it safe and escape it.
+        result.append("\\");
+        result.push_back(c);
         break;
       default:
         result.push_back(c);
         break;
     }
-
     prev = c;
   }
 
