@@ -120,7 +120,7 @@ int64_t ArrayOutputStream::ByteCount() const { return position_; }
 StringOutputStream::StringOutputStream(std::string* target) : target_(target) {}
 
 bool StringOutputStream::Next(void** data, int* size) {
-  ABSL_CHECK(target_ != NULL);
+  ABSL_CHECK(target_ != nullptr);
   size_t old_size = target_->size();
 
   // Grow the string.
@@ -148,13 +148,13 @@ bool StringOutputStream::Next(void** data, int* size) {
 
 void StringOutputStream::BackUp(int count) {
   ABSL_CHECK_GE(count, 0);
-  ABSL_CHECK(target_ != NULL);
+  ABSL_CHECK(target_ != nullptr);
   ABSL_CHECK_LE(static_cast<size_t>(count), target_->size());
   target_->resize(target_->size() - count);
 }
 
 int64_t StringOutputStream::ByteCount() const {
-  ABSL_CHECK(target_ != NULL);
+  ABSL_CHECK(target_ != nullptr);
   return target_->size();
 }
 
